@@ -27,7 +27,9 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#ifdef HAVE_UPOWER
 #include <upower.h>
+#endif
 
 #include "gsm-logout-dialog.h"
 #ifdef HAVE_SYSTEMD
@@ -52,13 +54,16 @@ typedef enum {
 struct _GsmLogoutDialogPrivate
 {
         GsmDialogLogoutType  type;
-
+#ifdef HAVE_UPOWER
         UpClient            *up_client;
+#endif
 #ifdef HAVE_SYSTEMD
         GsmSystemd          *systemd;
 #endif
         GsmConsolekit       *consolekit;
 
+        GtkWidget           *progressbar;
+
         int                  timeout;
         unsigned int         timeout_id;
 
@@ -147,9 +152,9 @@ gsm_logout_dialog_init (GsmLogoutDialog *logout_dialog)
         gtk_window_set_skip_taskbar_hint (GTK_WINDOW (logout_dialog), TRUE);
         gtk_window_set_keep_above (GTK_WINDOW (logout_dialog), TRUE);
         gtk_window_stick (GTK_WINDOW (logout_dialog));
-
+#ifdef HAVE_UPOWER
         logout_dialog->priv->up_client = up_client_new ();
-
+#endif
 #ifdef HAVE_SYSTEMD
         if (LOGIND_RUNNING())
             logout_dialog->priv->systemd = gsm_get_systemd ();
@@ -176,12 +181,12 @@ gsm_logout_dialog_destroy (GsmLogoutDialog *logout_dialog,
                 g_source_remove (logout_dialog->priv->timeout_id);
                 logout_dialog->priv->timeout_id = 0;
         }
-
+#ifdef HAVE_UPOWER
         if (logout_dialog->priv->up_client) {
                 g_object_unref (logout_dialog->priv->up_client);
                 logout_dialog->priv->up_client = NULL;
         }
-
+#endif
 #ifdef HAVE_SYSTEMD
         if (logout_dialog->priv->systemd) {
                 g_object_unref (logout_dialog->priv->systemd);
@@ -200,13 +205,37 @@ gsm_logout_dialog_destroy (GsmLogoutDialog *logout_dialog,
 static gboolean
 gsm_logout_supports_system_suspend (GsmLogoutDialog *logout_dialog)
 {
-        return up_client_get_can_suspend (logout_dialog->priv->up_client);
+        gboolean ret;
+        ret = FALSE;
+#ifdef HAVE_SYSTEMD
+        if (LOGIND_RUNNING())
+            ret = gsm_systemd_can_suspend (logout_dialog->priv->systemd);
+#endif
+#if defined(HAVE_SYSTEMD) && defined(HAVE_UPOWER)
+        else
+#endif
+#ifdef HAVE_UPOWER
+        ret = up_client_get_can_suspend (logout_dialog->priv->up_client);
+#endif
+        return ret;
 }
 
 static gboolean
 gsm_logout_supports_system_hibernate (GsmLogoutDialog *logout_dialog)
 {
-        return up_client_get_can_hibernate (logout_dialog->priv->up_client);
+        gboolean ret;
+        ret = FALSE;
+#ifdef HAVE_SYSTEMD
+        if (LOGIND_RUNNING())
+            ret = gsm_systemd_can_hibernate (logout_dialog->priv->systemd);
+#endif
+#if defined(HAVE_SYSTEMD) && defined(HAVE_UPOWER)
+        else
+#endif
+#ifdef HAVE_UPOWER
+        ret = up_client_get_can_hibernate (logout_dialog->priv->up_client);
+#endif
+        return ret;
 }
 
 static gboolean
@@ -273,7 +302,6 @@ gsm_logout_dialog_timeout (gpointer data)
         GsmLogoutDialog *logout_dialog;
         char            *seconds_warning;
         char            *secondary_text;
-        int              seconds_to_show;
         static char     *session_type = NULL;
         static gboolean  is_not_login;
 
@@ -286,35 +314,27 @@ gsm_logout_dialog_timeout (gpointer data)
                 return FALSE;
         }
 
-        if (logout_dialog->priv->timeout <= 30) {
-                seconds_to_show = logout_dialog->priv->timeout;
-        } else {
-                seconds_to_show = (logout_dialog->priv->timeout/10) * 10;
-
-                if (logout_dialog->priv->timeout % 10)
-                        seconds_to_show += 10;
-        }
-
         switch (logout_dialog->priv->type) {
         case GSM_DIALOG_LOGOUT_TYPE_LOGOUT:
                 seconds_warning = ngettext ("You will be automatically logged "
-                                            "out in %d second.",
+                                            "out in %d second",
                                             "You will be automatically logged "
-                                            "out in %d seconds.",
-                                            seconds_to_show);
+                                            "out in %d seconds",
+                                            logout_dialog->priv->timeout);
                 break;
 
         case GSM_DIALOG_LOGOUT_TYPE_SHUTDOWN:
                 seconds_warning = ngettext ("This system will be automatically "
-                                            "shut down in %d second.",
+                                            "shut down in %d second",
                                             "This system will be automatically "
-                                            "shut down in %d seconds.",
-                                            seconds_to_show);
+                                            "shut down in %d seconds",
+                                            logout_dialog->priv->timeout);
                 break;
 
         default:
                 g_assert_not_reached ();
         }
+        seconds_warning = g_strdup_printf (seconds_warning, logout_dialog->priv->timeout);
 
         if (session_type == NULL) {
 #ifdef HAVE_SYSTEMD
@@ -338,7 +358,7 @@ gsm_logout_dialog_timeout (gpointer data)
         }
 
         if (is_not_login) {
-                char *name, *tmp;
+                char *name;
 
                 name = g_locale_to_utf8 (g_get_real_name (), -1, NULL, NULL, NULL);
 
@@ -350,23 +370,26 @@ gsm_logout_dialog_timeout (gpointer data)
                         name = g_strdup (g_get_user_name ());
                 }
 
-                tmp = g_strdup_printf (_("You are currently logged in as \"%s\"."), name);
-                secondary_text = g_strconcat (tmp, "\n", seconds_warning, NULL);
-                g_free (tmp);
+                secondary_text = g_strdup_printf (_("You are currently logged in as \"%s\"."), name);
 
                 g_free (name);
         } else {
                 secondary_text = g_strdup (seconds_warning);
         }
 
+        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (logout_dialog->priv->progressbar),
+                                       logout_dialog->priv->timeout / 60.0);
+        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (logout_dialog->priv->progressbar),
+                                   seconds_warning);
+
         gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (logout_dialog),
                                                   secondary_text,
-                                                  seconds_to_show,
                                                   NULL);
 
         logout_dialog->priv->timeout--;
 
         g_free (secondary_text);
+        g_free (seconds_warning);
 
         return TRUE;
 }
@@ -395,6 +418,7 @@ gsm_get_dialog (GsmDialogLogoutType type,
 {
         GsmLogoutDialog *logout_dialog;
         GtkWidget       *dialog_image;
+        GtkWidget       *hbox;
         const char      *primary_text;
         const char      *icon_name;
 
@@ -475,6 +499,19 @@ gsm_get_dialog (GsmDialogLogoutType type,
 
         dialog_image = gtk_message_dialog_get_image (GTK_MESSAGE_DIALOG (logout_dialog));
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
+        hbox = gtk_hbox_new (FALSE, 0);
+#endif
+        logout_dialog->priv->progressbar = gtk_progress_bar_new ();
+        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (logout_dialog->priv->progressbar), 1.0);
+        gtk_box_pack_start (GTK_BOX (hbox),
+                            logout_dialog->priv->progressbar,
+                            TRUE, TRUE, 12);
+        gtk_widget_show_all (hbox);
+        gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (logout_dialog))), hbox);
+
         gtk_image_set_from_icon_name (GTK_IMAGE (dialog_image),
                                       icon_name, GTK_ICON_SIZE_DIALOG);
         gtk_window_set_icon_name (GTK_WINDOW (logout_dialog), icon_name);
